/*
 * Observable.java
 *
 * Created on 31 maggio 2005, 13.05
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package org.jaebi.midlet.util;

import java.util.Enumeration;
import java.util.Vector;

/**
 *
 * @author Antonio
 */
public class Observable {
    
    private boolean changed = false;
    private Vector obs;
    
    /** Creates a new instance of Observable */
    public Observable() {
        obs = new Vector();
    }
    
    /**
     * Aggiunge un nuovo Observer all'insieme di Observer per questo oggetto, a patto che l'Observer che si sta inserendo
     * non sia gi prensente nell'insieme.
     * Il metodo  dichiarato synchronized per evitare che, nel caso di multithreading, si verifichino race conditions che
     * portino ad inserire ugualmente due volte lo stesso Observer
     *
     * @param   o   an observer to be added.
     * @throws NullPointerException   if the parameter o is null.
     */
    public synchronized void addObserver(Observer o) {
        if (o == null)
            throw new NullPointerException();
        if (!obs.contains(o)) {
            obs.addElement(o);
        }
    }
    
    /**
     * Elimina un Observer dal'insieme.
     *
     * @param   o   the observer to be deleted.
     */
    public synchronized void deleteObserver(Observer o) {
        obs.removeElement(o);
    }
    
    /**
     * Se l'oggetto  cambiato (<code>hasChanged()</code> ritorna true), notifica tale cambiamento
     * agli obervers effettuando la chiamata al loro metodo <code>update</code>).
     * Una volta effettuata la notifica, viene chiamato il metodo <code>clearChanged</code>
     * per indicare che l'oggetto non  pi cambiato dall'ultima notifica.
     * @see     org.jaebi.client.midlet.util.Observable#clearChanged()
     * @see     org.jaebi.client.midlet.util.Observable#hasChanged()
     * @see     org.jaebi.client.midlet.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void notifyObservers() {
        notifyObservers(null);
    }
    
    /**
     * Se l'oggetto  cambiato (<code>hasChanged() ritorna true)</code>, notifica tale cambiamento
     * agli obervers effettuando la chiamata al loro metodo <code>update</code>.
     * Una volta effettuata la notifica, viene chiamato il metodo <code>clearChanged</code>
     * per indicare che l'oggetto non  pi cambiato dall'ultima notifica.
     *
     * @param   arg   any object.
     * @see     org.jaebi.client.midlet.util.Observable#clearChanged()
     * @see     org.jaebi.client.midlet.util.Observable#hasChanged()
     * @see     org.jaebi.client.midlet.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void notifyObservers(Object arg) {
        /*
         *Collezione degli observer usata come snapshot dello stato degli observer correnti
         */
        Enumeration observers;
        
        synchronized (this) {
            /*
             * Il codice incaricato di estrarre la lista degli observer da notificare
             * necessita di sincronizzazione ma la notifica vera e propria no (non dovrebbe)
             * Il peggior risultato di una qualsiasi race-condition che possa verificarsi qui (in assenza di sincronizzazione
             *   il seguente:
             * 1) a newly-added Observer will miss a
             *   notification in progress: siccome anche il metododo addObserver() 
             *   dichiarato sinchronized non pu verificarsi che un flusso di controllo aggiunga un nuovo observer
             *   mentre un altro sta recuperando la lista degli observer per effettuare la notifica
             * 2) a recently unregistered Observer will be
             *   wrongly notified when it doesn't care: non pu verificarsi che un flusso di controllo
             *   stia rimuovendo un observer mentre un altro sta recuperando
             *   la lista delgi observer a cui inviare la notifica (il metodo deleteObserver()  dichiarato anch'esso
             *   synchronized
             */
            if (!changed)
                return;
            observers = obs.elements();
            clearChanged();
        }
        
        while (observers.hasMoreElements())
            ((Observer)observers.nextElement()).update(this, arg);
           
    }
    
    /**
     * effettua il clear della lista degli observers
     */
    public synchronized void deleteObservers() {
        obs.removeAllElements();
    }
    
    /**
     * Marca questo oggetto <tt>Observable</tt> come changed
     */
    protected synchronized void setChanged() {
        changed = true;
    }
    
    /**
     * Indica che gli observer sono stati notificati dell'ultimo cambiamento.
     * il metodo <tt>hasChanged</tt> ora ritorner <tt>false</tt>.
     * Questo metodo  chiamato automaticamente dal metodo
     * <code>notifyObservers</code>.
     *
     * @see     org.jaebi.client.midlet.util.Observable#notifyObservers()
     * @see     org.jaebi.client.midlet.util.Observable#notifyObservers(java.lang.Object)
     */
    protected synchronized void clearChanged() {
        changed = false;
    }
    
    /**
     * Verifica se l'oggetto  cambiato.
     *
     * @return  <code>true</code> se e solo se il metodo <code>setChanged</code>
     *           stato chiamato pi recetemente di quando  stato chiamato
     *          <code>clearChanged</code> su quest'oggetto;
     *          <code>false</code> altrimenti.
     * @see     org.jaebi.client.midlet.util.Observable#clearChanged()
     * @see     org.jaebi.client.midlet.util.Observable#setChanged()
     */
    public synchronized boolean hasChanged() {
        return changed;
    }
    
    /**
     * Ritorna il numero di observer per questo oggetto
     *
     * @return  il numero di observer per questo oggetto.
     */
    public synchronized int countObservers() {
        return obs.size();
    }
}
